#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/select.h>

#include "capi.h"

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(x)	sizeof(x)/sizeof(x[0])
#endif

#ifndef min
#define min(a,b)	((a) < (b) ? (a) : (b))
#endif


//capi_get_int8
//capi_get_int16
//capi_get_ip4addr
//capi_get_macaddr
//capi_get_string
//capi_get_buffer

char *capi_get_hexstring(char *buf)
{
	char out[strlen(buf)];
	char *obuf;
	int i = 0;
	int j = 0;


	while (buf[i] != '\0') {
		if (strstr(&buf[i], "0x") == &buf[i])
			i += 2;
		else if (buf[i] == ' ' || buf[i] == '{' || buf[i] == '}')
			i++;
		else if (isxdigit(buf[i]))
			out[j++] = buf[i++];
		else {
			printf("Error in string!\n");
			return NULL;
		}
	}

	out[j++] = '\0';
	obuf = malloc(j * sizeof(char));
	if (!obuf)
		return NULL;

	strcpy(obuf, out);

	return obuf;
}

int capi_parse(const struct capi_policy *policy, int num_policy,
	       struct capi_attribute **attr, char *args, int len)
{
	char *token = NULL;
	char *data = args;
	int expect_value = 0;
	int iidx[num_policy];
	int i;


	for (i = 0; i < num_policy; i++) {
		int num = policy[i].maxnum ? policy[i].maxnum : 1;

		attr[i] = calloc(num, sizeof(struct capi_attribute));
		if (!attr[i]) {
			while (--i >= 0)
				free(attr[i]);
			return -1;
		}

		attr[i]->attr = NULL;
		attr[i]->len = 0;
		iidx[i] = 0;
	}


	for (;;) {
		token = strsep(&data, ",");
		if (token == NULL || token[0] == '\0')
			break;

		/*
		printf(".................................\n");
		printf("token: '%s' { %s}\n", token, expect_value ? "value" : "param");
		printf("rem: '%s'\n", data);
		printf(".................................\n");
		*/

		if (!expect_value) {
			int found = 0;

			i = 0;

			for (i = 0; i < num_policy; i++) {
				char *tmp;
				size_t x;

				tmp = strstr(policy[i].name, "*");
				x = strlen(policy[i].name);
				if (tmp)
					x -= strlen(tmp);

				if (!strncasecmp(policy[i].name, token, x)) {
					found = 1;
					attr[i][iidx[i]].type = policy[i].type;
					attr[i][iidx[i]].attr = token;
					fprintf(stderr, "param: attr[%d][%d] = %s\n",
						i, iidx[i], attr[i][iidx[i]].attr);
					break;
				}
			}

			if (!found)
				return -1;

		} else {
			attr[i][iidx[i]].len = strlen(token);
			attr[i][iidx[i]].value = token;
			fprintf(stderr, "value: attr[%d][%d] = %s\n",
				i, iidx[i], attr[i][iidx[i]].value);
			iidx[i]++;
		}

		expect_value = !expect_value;
	}

	if (expect_value)
		return -1;

	return 0;
}

struct capi_cmd *capi_lookup_cmd(struct capi *ctx, char *cmdname)
{
	struct capi_cmd *c = ctx->cmdtable;
	int i;

	printf("strlen: %zu\n", strlen(cmdname));

	for (i = 0; i < ctx->num_cmds; i++) {
		if (!strcasecmp(cmdname, c[i].name)) {
			printf("matched: c->name = %s\n", c[i].name);
			return &c[i];
		}
	}

	return NULL;
}

char *capi_extract_cmd(char *cmdline, char **args)
{
	*args = NULL;
	return strtok_r(cmdline, ",", args);
}

int capi_init_cmdtable(struct capi *ctx, struct capi_cmd *table, int num_cmds)
{
	if (ctx && table && num_cmds > 0) {
		ctx->cmdtable = calloc(num_cmds, sizeof(struct capi_cmd));
		if (!ctx->cmdtable)
			return -1;

		printf("%s: table = %p\n", __func__, table);

		memcpy(ctx->cmdtable, table, num_cmds * sizeof(struct capi_cmd));
		ctx->num_cmds = num_cmds;
		return 0;
	}

	return -1;
}

int capi_validate_cmd(struct capi *ctx, struct capi_client *cli, char *cmdline)
{
	char *cmdname, *args;
	struct capi_cmd *cmd;
	int ret = 0;


	if (!cmdline || cmdline[0] == '\0')
		return -1;

	cmdname = capi_extract_cmd(cmdline, &args);
	if (!cmdname)
		return -1;

	cmd = capi_lookup_cmd(ctx, cmdname);
	if (!cmd)
		return -1;

	if (args == NULL)
		args = "";

	if (cmd->validate)
		ret = cmd->validate(cmd, args);

	return ret;
}

int capi_run_cmd(struct capi *ctx, struct capi_client *cli, char *cmdline,
		 char *response)
{
	char *cmdname, *args;
	struct capi_cmd *cmd;
	int ret = 0;


	fprintf(stderr, "%s: cmdline = %s\n", __func__, cmdline);

	if (!cmdline || cmdline[0] == '\0')
		return -1;

	cmdname = capi_extract_cmd(cmdline, &args);
	if (!cmdname)
		return -1;

	cmd = capi_lookup_cmd(ctx, cmdname);
	if (!cmd)
		return -1;

	if (args == NULL)
		args = "";

	fprintf(stderr, "%s: arguments: %s\n", __func__, args);
	if (cmd->handler)
		ret = cmd->handler(cmd, args, response);

	return ret;
}
