#ifndef CAPI_H
#define CAPI_H

#include <stdint.h>

#include <libubox/list.h>

#define CAPI_BACKLOG	3

#define CAPI_CMD_SIZE 4096
#define CAPI_RESP_SIZE 2048

enum capi_datatype {
	CAPI_UNDEF,
	CAPI_INT8,
	CAPI_BOOLEAN = CAPI_INT8,
	CAPI_INT16,
	CAPI_HEXSTRING,
	CAPI_IP4ADDR,
	CAPI_IP6ADDR,
	CAPI_MACADDR,
	CAPI_STRING,
	CAPI_LIST,
	CAPI_DATATYPE_NUM,
};


enum capi_state {
	CAPI_READY,
	CAPI_RUNNING,
	CAPI_INVALID,
	CAPI_ERROR,
	CAPI_COMPLETE,
};

struct capi_cmdline {
	size_t len;
	char *cmdline;
	struct list_head list;
};

struct capi_policy {
	char name[256];
	enum capi_datatype type;
	int maxnum;
};

struct capi_attr {
	char *param;
	size_t len;
	char *value;
};

struct capi_attribute {
	enum capi_datatype type;
	char *attr;
	size_t len;
	char *value;
};


#define CAPI_CMD(c)                     { #c, validate_ ## c, handle_ ## c }
#define capi_handle(c, m, a, r)		handle_ ## c(m, a, r)
#define capi_validate(c, m, s)          validate_ ## c(m, s)

struct capi_cmd {
	char name[256];
	int (*validate)(struct capi_cmd *, char *cmdline);
	int (*handler)(struct capi_cmd *, char *cmdline, char *response);
};


struct capi_client {
	enum capi_state state;
	struct list_head cmdlinelist;
	struct list_head list;

	/* other client specific data */
	int sk;
	uint16_t port;
};

struct capi_program {
	char name[64];
	struct capi_cmd *cmdtable;
	int num_cmds;
	struct list_head list;
};

struct capi {
	struct list_head clientlist;
	struct capi_cmd *cmdtable;
	//struct list_head proglist;	//TODO
	int num_cmds;
	int sk;
	uint16_t port;
	fd_set rfds, wfds;
	int nfds;
};



char *capi_get_hexstring(char *buf);

int capi_init_cmdtable(struct capi *ctx, struct capi_cmd *table, int num_cmds);

char *capi_extract_cmd(char *cmdline, char **args);

struct capi_cmd *capi_lookup_cmd(struct capi *ctx, char *cmdname);

int capi_validate_cmd(struct capi *ctx, struct capi_client *cli, char *cmdline);
int capi_run_cmd(struct capi *ctx, struct capi_client *cli, char *cmdline,
		 char *response);

int capi_parse(const struct capi_policy *policy, int num_policy,
	       struct capi_attribute **attr, char *args, int len);



#define capi_register(name, cmds)				  \
extern int capi_register_ ## name(struct capi *);		  \
int capi_register_ ## name(struct capi *__ctx)			  \
{								  \
	return capi_init_cmdtable(__ctx, cmds, ARRAY_SIZE(cmds)); \
}

#define capi_call(name)		capi_register_ ## name(ctx)

#endif /* CAPI_H */
