#include <errno.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>

#include <easy/easy.h>

#include "capi.h"

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(x)	sizeof(x)/sizeof(x[0])
#endif


static const struct capi_policy pol_DEV_SET_CONFIG[] = {
	[0] = { .name = "Backhaul", .type = CAPI_STRING },
	[1] = { .name = "BSS_Info*", .type = CAPI_STRING, .maxnum = 16 },
	[2] = { .name = "Name", .type = CAPI_STRING },
	[3] = { .name = "Program", .type = CAPI_STRING },
};

static int set_bssinfoX(char *bssinfo, char *args, int args_len)
{
	char *conf = " -b {\\\"config\\\":\\\"mapcontroller\\\","
			  "\\\"type\\\":\\\"ap\\\",\\\"values\\\":{"
					"\\\"band\\\":\\\"%d\\\""
					",\\\"ssid\\\":\\\"%s\\\""
					",\\\"encryption\\\":\\\"%s\\\""
					",\\\"multi_ap\\\":\\\"%d\\\""
					",\\\"key\\\":\\\"%s\\\""
					",\\\"vid\\\":\\\"1\\\"}}";

	const char *auth = "psk2";
	const char *ssid = NULL;
	const char *enc = NULL;
	const char *key = NULL;
	int multiap = 0;
	int band = 5;
	int pos = 0;
	char *token;
	int ret;
	char *in = strdup(bssinfo);

	if (!in)
		return -1;

	printf("%s\n", in);

	for (;;) {
		token = strsep(&in, " ");
		if (token == NULL)
			break;

		switch (pos) {
		case 0:
			printf("1905 ALID: %s\n", token);
			break;
		case 1:
		{
			int opclass = atoi(token);

			printf("opclass: %d\n", atoi(token));
			if (opclass == 81 || opclass == 83 || opclass == 84)
				band = 2;

			/* FIXME: 5gh, 6g */
			break;
		}
		case 2:
			printf("ssid: %s\n", token);
			ssid = token;
			break;
		case 3:
			printf("auth: %s\n", token);
			if (!strncmp(token, "0x0020", strlen("0x0020")))
				auth = "psk2";
			break;
		case 4:
			printf("enc: %s\n", token);
			if (!strncmp(token, "0x0008", strlen("0x0008")))
				enc = "aes";
			break;
		case 5:
			printf("key: %s\n", token);
			key = token;
			break;
		case 6:
			printf("bBSS: %s\n", token);
			if (atoi(token) == 1)
				multiap |= 1;

			break;
		case 7:
			printf("fBSS: %s\n", token);
			if (atoi(token) == 1)
				multiap |= 2;
			break;
		default:
			break;
		}

		pos++;
	}

	/* TODO: delete matching config sections if ALID and opclass, but
	 * nothing else, is only provided.
	 */

	ret = snprintf(args, args_len, conf, band, ssid, enc, multiap, key);

	free(in);
	return ret;
}

#define ADD_PARAM(buf_ptr, buf_len, ...)			\
do {								\
	int _len = snprintf(buf_ptr, buf_len, __VA_ARGS__);	\
	if (_len >= 0) {					\
		buf_len -= _len;				\
		buf_ptr += _len;				\
	} else {						\
		fprintf(stderr, "snprintf FAIL %d\n", _len); 	\
		return _len;					\
	}							\
} while (0)

int capi_handle(DEV_SET_CONFIG, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_SET_CONFIG)];
	char buf[4096] = {0};
	char out[1024] = {0};
	char *buf_ptr = buf;
	int buf_len = sizeof(buf) - 1;
	int i, ret;

	ret = capi_parse(pol_DEV_SET_CONFIG, ARRAY_SIZE(pol_DEV_SET_CONFIG),
			 tb, args, strlen(args));
	if (ret < 0)
		return ret;

	if (tb[0][0].attr) {
		if (!strncasecmp(tb[0][0].value, "Eth", strlen("Eth"))) {
			ADD_PARAM(buf_ptr, buf_len, " -r Eth ");
		} else {
			char *bsta;
			uint8_t bmac[6] = {0};

			bsta = capi_get_hexstring(tb[0][0].value);
			strtob(bsta, 6, bmac);
			ADD_PARAM(buf_ptr, buf_len, " -r " MACFMT , MAC2STR(bmac));
		}
	}

	if (tb[2][0].attr) {
		fprintf(stderr, "Device name = %s\n", tb[2][0].value);
		ADD_PARAM(buf_ptr, buf_len, " -n %s ", tb[2][0].value);
	}

	if (tb[3][0].attr) {
		if (strncasecmp(tb[3][0].value, "MAP", strlen("MAP"))) {
			fprintf(stderr, "Unsupported program = %s\n", tb[3][0].value);
			return -1;
		}
	}

	if (tb[1][0].attr) {
		for (i = 0; i < pol_DEV_SET_CONFIG[1].maxnum && tb[1][i].attr; i++) {
			fprintf(stderr, "BSS_Info = %s\n", tb[1][i].value);

			ret = set_bssinfoX(tb[1][i].value, buf_ptr, buf_len);
			if (ret < 0)
				return ret;

			buf_ptr += ret;
			buf_len -= ret;
		}
	}

	return Cmd(out, sizeof(out), "/lib/wifi/capi/dev_set_config %s", buf);
}

int capi_validate(DEV_SET_CONFIG, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_SET_CONFIG)];
	int ret;

	ret = capi_parse(pol_DEV_SET_CONFIG, ARRAY_SIZE(pol_DEV_SET_CONFIG),
			 tb, args, strlen(args));

	return ret;
}


static const struct capi_policy pol_DEV_SEND_1905[] = {
	[0] = { .name = "DestALID", .type = CAPI_MACADDR },
	[1] = { .name = "MessageTypeValue", .type = CAPI_STRING },
	[2] = { .name = "TLV_Type", .type = CAPI_INT8, .maxnum = 32 },
	[3] = { .name = "TLV_Length", .type = CAPI_INT16, .maxnum = 32 },
	[4] = { .name = "TLV_Value", .type = CAPI_HEXSTRING, .maxnum = 32 },
};

int capi_handle(DEV_SEND_1905, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_SEND_1905)];
	char buf[4096] = "ubus call ieee1905 cmdu '{";
	char out[512] = {0};
	bool ok = false;
	int ret;
	char *v;
	int i;


	ret = capi_parse(pol_DEV_SEND_1905, ARRAY_SIZE(pol_DEV_SEND_1905),
			 tb, args, strlen(args));

	if (tb[0][0].attr)
		snprintf(buf + strlen(buf), sizeof(buf) -1, "\"dst\":\"%s\"", tb[0][0].value);

	if (tb[1][0].attr) {
		char *type;

		type = capi_get_hexstring(tb[1][0].value);
		snprintf(buf + strlen(buf), sizeof(buf) -1, ",\"type\":%s", type);
	}

	if (tb[2][0].attr) {
		snprintf(buf + strlen(buf), sizeof(buf) -1, ",\"data\":\"");

		for (i = 0; i < pol_DEV_SEND_1905[2].maxnum && tb[2][i].attr; i++) {
			char *tag;
			char *len;
			char *data;

			tag = capi_get_hexstring(tb[2][i].value);
			len = capi_get_hexstring(tb[3][i].value);
			data = capi_get_hexstring(tb[4][i].value);

			fprintf(stderr, "tag = %s  len = %s  data = %s\n", tag, len, data);

			snprintf(buf + strlen(buf), sizeof(buf) -1, "%s%s%s", tag, len, data);
		}

		snprintf(buf + strlen(buf), sizeof(buf) -1, "\"");
	}

	snprintf(buf + strlen(buf), sizeof(buf) -1, "}'");

	fprintf(stderr, "UBUS: %s\n", buf);
	chrCmd(out, sizeof(out), "%s", buf);
	//fprintf(stderr, "%s\n", out);

	if (strcasestr(out, "\"ok\""))
		ok = true;

	if (ok) {
		v = strcasestr(out, "\"mid\": ");
		if (v) {
			v += strlen("\"mid\": ");
			sprintf(response, "MID,%d", atoi(v));
		}
	} else {
		response = NULL;
	}

	return ret;
}

int capi_validate(DEV_SEND_1905, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_SEND_1905)];
	int ret;


	ret = capi_parse(pol_DEV_SEND_1905, ARRAY_SIZE(pol_DEV_SEND_1905),
			 tb, args, strlen(args));

	return ret;
}


struct capi_policy pol_DEV_RESET_DEFAULT[] = {
	[0] = { .name = "DevRole", .type = CAPI_STRING },
	[1] = { .name = "Name", .type = CAPI_STRING },
	[2] = { .name = "Program", .type = CAPI_STRING },
	[3] = { .name = "Type", .type = CAPI_STRING },
};

int capi_handle(DEV_RESET_DEFAULT, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_RESET_DEFAULT)];
	char buf[4096] = {0};
	char out[1024] = {0};
	char *buf_ptr = buf;
	int buf_len = sizeof(buf) - 1;
	int ret;


	ret = capi_parse(pol_DEV_RESET_DEFAULT, ARRAY_SIZE(pol_DEV_RESET_DEFAULT),
			 tb, args, strlen(args));
	if (ret < 0)
		return ret;

	if (tb[0][0].attr) {
		if (!strncasecmp(tb[0][0].value, "Controller", strlen("Controller"))) {
			fprintf(stderr, "Device Role = Controller\n");
			ADD_PARAM(buf_ptr, buf_len, " -r Controller ");
			setenv("CAPI_DEV_ROLE", "Controller", 1);
		} else if (!strncasecmp(tb[0][0].value, "Agent", strlen("Agent"))) {
			fprintf(stderr, "Device Role = Agent\n");
			ADD_PARAM(buf_ptr, buf_len, " -r Agent ");
			setenv("CAPI_DEV_ROLE", "Agent", 1);
		}
	}

	if (tb[1][0].attr) {
		fprintf(stderr, "Device name = %s\n", tb[1][0].value);
		ADD_PARAM(buf_ptr, buf_len, " -n %s ", tb[1][0].value);
	}

	if (tb[3][0].attr) {
		fprintf(stderr, "Device Type = %s\n", tb[3][0].value);
		ADD_PARAM(buf_ptr, buf_len, " -t %s ", tb[3][0].value);
	}

	return Cmd(out, sizeof(out), "/lib/wifi/capi/dev_reset_default %s", buf);
}

int capi_validate(DEV_RESET_DEFAULT, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_RESET_DEFAULT)];
	int ret;

	ret = capi_parse(pol_DEV_RESET_DEFAULT, ARRAY_SIZE(pol_DEV_RESET_DEFAULT),
			 tb, args, strlen(args));

	return ret;
}


struct capi_policy pol_START_WPS_REGISTRATION[] = {
	[0] = { .name = "Band", .type = CAPI_STRING },
	[1] = { .name = "Interface", .type = CAPI_STRING },
	[2] = { .name = "Name", .type = CAPI_STRING },
	[3] = { .name = "SSID", .type = CAPI_STRING },
	[4] = { .name = "WPSConfigMethod", .type = CAPI_STRING },
	[5] = { .name = "WPSRole", .type = CAPI_STRING },
};

int capi_handle(START_WPS_REGISTRATION, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_START_WPS_REGISTRATION)];
	char buf[1024] = {0};
	char out[512] = {0};
	const char *band = NULL;
	const char *ifname = NULL;
	const char *method = NULL;
	const char *role = NULL;
	char *buf_ptr = buf;
	int buf_len = sizeof(buf) - 1;
	int ret;


	ret = capi_parse(pol_START_WPS_REGISTRATION,
			 ARRAY_SIZE(pol_START_WPS_REGISTRATION),
			 tb, args, strlen(args));
	if (ret < 0)
		return ret;


	if (tb[0][0].attr) {
		band = tb[0][0].value;
		fprintf(stderr, "Band = %s\n", band);
	}

	if (tb[1][0].attr) {
		ifname = tb[1][0].value;

		fprintf(stderr, "Interface = %s\n", ifname);
		ADD_PARAM(buf_ptr, buf_len, " -i %s", ifname);
	}

	if (tb[2][0].attr) {
		fprintf(stderr, "Name = %s\n", tb[2][0].value);
	}

	if (tb[3][0].attr) {
		fprintf(stderr, "SSID = %s\n", tb[3][0].value);
	}

	if (tb[4][0].attr) {
		method = tb[4][0].value;

		fprintf(stderr, "WPSConfigMethod = %s\n", method);
		if (strncasecmp(method, "pbc", 3) &&
		    strncasecmp(method, "pin", 3)) {
			return -1;
		}

		ADD_PARAM(buf_ptr, buf_len, " -m %s", method);
		/* method = pbc (default) */
	}

	if (tb[5][0].attr) {
		role = tb[5][0].value;

		fprintf(stderr, "WPSRole = %s\n", role);
		if (strncasecmp(role, "enrollee", strlen("enrollee")))
			ADD_PARAM(buf_ptr, buf_len, " -r enrollee ");
		else if (strncasecmp(role, "registrar", strlen("registrar")))
			ADD_PARAM(buf_ptr, buf_len, " -r registrar ");
		else
			return -1;
	}

	chrCmd(out, sizeof(out), "/lib/wifi/capi/start_wps_registration %s", buf);
	fprintf(stderr, "out: %s (len = %zd)\n", out, strlen(out));
	if (strlen(out))
		strcpy(response, out);

	return 0;
}

int capi_validate(START_WPS_REGISTRATION, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_START_WPS_REGISTRATION)];
	int ret;

	ret = capi_parse(pol_START_WPS_REGISTRATION,
			 ARRAY_SIZE(pol_START_WPS_REGISTRATION),
			 tb, args, strlen(args));

	return ret;
}

struct capi_policy pol_AP_DEAUTH_STA[] = {
	[0] = { .name = "Name", .type = CAPI_STRING },
	[1] = { .name = "STA_MAC_Address", .type = CAPI_MACADDR },
};

int capi_handle(AP_DEAUTH_STA, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_AP_DEAUTH_STA)];
	char buf[128] = {0};
	char out[512] = {0};
	int ret;


	ret = capi_parse(pol_AP_DEAUTH_STA, ARRAY_SIZE(pol_AP_DEAUTH_STA),
			 tb, args, strlen(args));

	if (tb[0][0].attr) {
		fprintf(stderr, "Name = %s\n", tb[0][0].value);
	}

	if (tb[1][0].attr) {
		fprintf(stderr, "STA Macaddress = %s\n", tb[1][0].value);
		snprintf(buf + strlen(buf), sizeof(buf) -1, "%s", tb[1][0].value);
	}

	chrCmd(out, sizeof(out), "/lib/wifi/capi/ap_deauth_sta %s", buf);
	fprintf(stderr, "out: %s (len = %zd)\n", out, strlen(out));

	return ret;
}

int capi_validate(AP_DEAUTH_STA, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_AP_DEAUTH_STA)];
	int ret;

	ret = capi_parse(pol_AP_DEAUTH_STA, ARRAY_SIZE(pol_AP_DEAUTH_STA),
			 tb, args, strlen(args));

	return ret;
}


/*
 * 10.1 CA_GET_VERSION
 */

struct capi_policy pol_CA_GET_VERSION[] = {
	[0] = { .name = "TestInfo", .type = CAPI_STRING },
};

int capi_handle(CA_GET_VERSION, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_CA_GET_VERSION)];
	char *test_info = "";
	char out[1024] = {0};
	int ret;

	ret = capi_parse(pol_CA_GET_VERSION, ARRAY_SIZE(pol_CA_GET_VERSION),
			 tb, args, strlen(args));
	if (ret < 0)
		return ret;

	if (tb[0][0].attr) {
		fprintf(stderr, "TestInfo = %s\n", tb[0][0].value);
		test_info = tb[0][0].value;
	}

	// TODO handle TestInfo

	chrCmd(out, sizeof(out), "opkg info wfa-capi | grep Version: | cut -d ' ' -f2");
	fprintf(stderr, "out: %s (len = %zd)\n", out, strlen(out));

	snprintf(response, CAPI_RESP_SIZE - 1, "version,%s", out);

	return 0;
}

int capi_validate(CA_GET_VERSION, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_CA_GET_VERSION)];
	int ret;

	ret = capi_parse(pol_CA_GET_VERSION, ARRAY_SIZE(pol_CA_GET_VERSION),
			 tb, args, strlen(args));

	return ret;
}


/*
 * 10.5 DEVICE_GET_INFO
 */

struct capi_policy pol_DEVICE_GET_INFO[] = {
	[0] = { .name = "Name", .type = CAPI_STRING },
};

int capi_handle(DEVICE_GET_INFO, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEVICE_GET_INFO)];
	char *name = "";
	char out[2047] = {0};
	int ret;

	ret = capi_parse(pol_DEVICE_GET_INFO, ARRAY_SIZE(pol_DEVICE_GET_INFO),
			 tb, args, strlen(args));

	if (ret < 0)
		return ret;

	if (tb[0][0].attr) {
		fprintf(stderr, "Name = %s\n", tb[0][0].value);
		name = tb[0][0].value;
	}

	chrCmd(out, sizeof(out), "/lib/wifi/capi/device_get_info %s", name);
	fprintf(stderr, "out: %s (len = %zd)\n", out, strlen(out));

	snprintf(response, CAPI_RESP_SIZE - 1, "%s", out);

	return 0;
}

int capi_validate(DEVICE_GET_INFO, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEVICE_GET_INFO)];
	int ret;

	ret = capi_parse(pol_DEVICE_GET_INFO, ARRAY_SIZE(pol_DEVICE_GET_INFO),
			 tb, args, strlen(args));

	return ret;
}


static const struct capi_policy pol_DEV_GET_PARAMETER[] = {
	[0] = { .name = "Parameter", .type = CAPI_STRING },
	[1] = { .name = "Program", .type = CAPI_STRING },
	[2] = { .name = "RUID", .type = CAPI_MACADDR },
	[3] = { .name = "SSID", .type = CAPI_STRING },
};

int capi_handle(DEV_GET_PARAMETER, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_GET_PARAMETER)];
	const char *parameter = NULL;
	const char *ruid = NULL;
	const char *ssid = NULL;
	char out[1024] = {0};
	char buf[1024] = {0};
	char *buf_ptr = buf;
	int buf_len = sizeof(buf) - 1;
	int ret;


	ret = capi_parse(pol_DEV_GET_PARAMETER,
			 ARRAY_SIZE(pol_DEV_GET_PARAMETER),
			 tb, args, strlen(args));
	if (ret < 0)
		return ret;

	if (tb[0][0].attr) {
		parameter = tb[0][0].value;

		fprintf(stderr, "Parameter = %s\n", parameter);
		if (strncasecmp(parameter, "MacAddr", 7) &&
		    strncasecmp(parameter, "BSSID", 5) &&
		    strncasecmp(parameter, "ALID", 4)) {
			return -1;
		}

		ADD_PARAM(buf_ptr, buf_len, " -p %s", parameter);
	}

	if (tb[1][0].attr) {
		fprintf(stderr, "Program = %s\n", tb[1][0].value);
		if (strncasecmp(tb[1][0].value, "MAP", 3))
				return -1;

		/* unused(tb[1][0].value); */
	}

	if (tb[2][0].attr) {
		ruid = tb[2][0].value;

		fprintf(stderr, "RUID = %s\n", ruid);
		snprintf(buf + strlen(buf), sizeof(buf) -1, " -r %s", ruid);
	}

	if (tb[3][0].attr) {
		ssid = tb[3][0].value;

		fprintf(stderr, "SSID = %s\n", ssid);
		snprintf(buf + strlen(buf), sizeof(buf) -1, " -s %s", ssid);
	}

	chrCmd(out, sizeof(out), "/lib/wifi/capi/dev_get_parameter %s", buf);
	fprintf(stderr, "out: %s (len = %zd)\n", out, strlen(out));

	snprintf(response, CAPI_RESP_SIZE - 1, "%s,%s", parameter, out);
	return 0;
}

int capi_validate(DEV_GET_PARAMETER, struct capi_cmd *cmd, char *args)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_GET_PARAMETER)];
	int ret;

	ret = capi_parse(pol_DEV_GET_PARAMETER,
			 ARRAY_SIZE(pol_DEV_GET_PARAMETER),
			 tb, args, strlen(args));

	return ret;
}


static const struct capi_policy pol_DEV_SET_RFEATURE[] = {
	[0] = { .name = "Name", .type = CAPI_STRING },
	[1] = { .name = "Type", .type = CAPI_STRING },
	[2] = { .name = "Bssid", .type = CAPI_STRING },
	[3] = { .name = "Assoc_Disallow", .type = CAPI_STRING },
};

int capi_handle(DEV_SET_RFEATURE, struct capi_cmd *cmd, char *args,
		char *response)
{
	struct capi_attribute *tb[ARRAY_SIZE(pol_DEV_SET_RFEATURE)];
	char buf[4096] = {0};
	char out[1024] = {0};
	char *buf_ptr = buf;
	int buf_len = sizeof(buf) - 1;
	int i, ret;

	ret = capi_parse(pol_DEV_SET_RFEATURE, ARRAY_SIZE(pol_DEV_SET_RFEATURE),
			 tb, args, strlen(args));
	if (ret < 0)
		return ret;

	if (tb[2][0].attr) {
		const char *macstr;

		macstr = tb[2][0].value;
		ADD_PARAM(buf_ptr, buf_len, " -b %s", macstr);
	}

	if (tb[3][0].attr) {
		if (!strncasecmp(tb[3][0].value, "Enable", strlen("Enable"))) {
			ADD_PARAM(buf_ptr, buf_len, " -x %d ", 1);
		} else {
			ADD_PARAM(buf_ptr, buf_len, " -x %d ", 0);
		}
	}

	return Cmd(out, sizeof(out), "/lib/wifi/capi/dev_set_rfeature %s", buf);
}

int capi_validate(DEV_SET_RFEATURE, struct capi_cmd *cmd, char *args)
{
	//TODO
	return 0;
}

static struct capi_cmd cmds[] = {
	CAPI_CMD(CA_GET_VERSION),
	CAPI_CMD(DEV_SET_CONFIG),
	CAPI_CMD(DEV_SEND_1905),
	CAPI_CMD(DEV_RESET_DEFAULT),
	CAPI_CMD(START_WPS_REGISTRATION),
	CAPI_CMD(AP_DEAUTH_STA),
	CAPI_CMD(DEVICE_GET_INFO),
	CAPI_CMD(DEV_GET_PARAMETER),
	CAPI_CMD(DEV_SET_RFEATURE),
	/* CAPI_CMD(DEV_SEND_FRAME), */	//TODO
};

#if 0	//TODO
static struct capi_program map = {
	.name = "map",
	.cmdtable = cmds,
	.num_cmds = ARRAY_SIZE(cmds)
};
#endif

capi_register(easymesh, cmds);
