#!/bin/sh

. /lib/functions.sh
. /usr/share/libubox/jshn.sh


# Usage: wireless_get_band_from_ifname wlan0
wireless_get_band_from_ifname() {
	band=$(ubus -t 2 call wifi.radio.$1 status | grep '"band":' | cut -f 2 -d ':' | tr -d '", ')
	echo "$band"
}

# Usage: wireless_get_ifname_from_band 24G
wireless_get_ifname_from_band() {
	local b=$1
	local ifname=""

	robj=$(ubus -t 2 list wifi.radio.*)

	for i in $robj; do {
		band=$(ubus -t 2 call $i status | grep '"band":' | cut -f 2 -d ':' | tr -d '",. ')
		band=${band::-2}
		[ "$band" == "$b" ] && {
			ifname="${i/wifi.radio./}"
			echo "$ifname"
			return
		}
	}; done

	echo "$ifname"
}

# Usage: reset_wireless_config
reset_wireless_config() {
	config_load wireless

	reset_config() {
		local sec=$1

		config_get ifname $sec ifname
		config_get mode $sec mode
		config_get multi_ap $sec multi_ap

		[ -z "$multi_ap" ] && return

		[ "$mode" == "ap" ] && {
                        uci -q delete wireless.${sec}
                }

                [ "$mode" == "sta" ] && {
                        uci -q delete wireless.${sec}.ssid
                        uci -q delete wireless.${sec}.key
                }
        }

        config_foreach reset_config wifi-iface
        uci commit wireless
}

# Usage: reset_mapagent_config -b wlan1
reset_mapagent_config() {
        config_load mapagent

	[ $# -gt 0 ] && {
		while [ "$1" != "" ]; do
                        case $1 in
                                -b | --backhaul )
                                shift
                                bsta_ifname=$1
                                ;;
                        * )
                                ;;
                        esac
                        shift
                done
        }

	delete_section() {
                local sec=$1

                uci -q delete mapagent.${sec}
        }

        uci -q delete mapagent.agent.controller_macaddr
        config_foreach delete_section fh-iface

        [ -n "$bsta_ifname" ] && {
                band=$(wireless_get_band_from_ifname $bsta_ifname)
                [ $band == "5GHz" ] && b=5 || b=2

                set_backhaul() {
                        local sec=$1
                        local bsta=$2

                        config_get device $sec device
                        config_get dedicated_backhaul $sec dedicated_backhaul

                        [ "$device" == "$bsta" ] && {
                                uci -q set mapagent.${sec}.device=$bsta
                                uci -q set mapagent.${section}.band=$b
                        } || {
                                uci -q delete mapagent.${sec}.dedicated_backhaul
                        }
                }

                config_foreach set_backhaul wifi-radio $bsta_ifname
        }

        uci commit mapagent
}
